//================================================================================================
// $Id: checker.cpp 5131 2016-12-26 20:34:09Z aron $
//
// Project:
//		Welltype Parser
// Contents:
//		AST checker
// Authors:
//		Aron Barath, 2014-2016
//================================================================================================
// Copyright (c) Aron Barath
//================================================================================================

#include "checker.h"
#include "astnode.h"
#include "astvisitor.h"
#include "ast/ast_functioncall.h"
#include "ast/ast_functionpointercall.h"
#include "ast/ast_functiondefinition.h"
#include "ast/ast_functionsignature.h"
#include "ast/ast_variableaccess.h"
#include "ast/ast_exceptioncompound.h"
#include "ast/ast_statement.h"
#include "ast/ast_compound.h"
#include "ast/ast_ifstatement.h"

namespace wtparser
{
namespace checker
{

namespace visitor
{

class PurePropVisitor : public wtast::AstVisitor
{
	bool mPure;

public:
	PurePropVisitor()
		: wtast::AstVisitor()
		, mPure(true)
	{
		// ok
	}

	bool IsPure() const
	{
		return mPure;
	}

	bool Visit(wtast::FunctionCall* fcall)
	{
		if(mPure && !fcall->GetFunctionSignature()->IsPure())
		{
			mPure = false;
		}

		return mPure;
	}

	bool Visit(wtast::FunctionPointerCall* fpcall)
	{
		if(mPure && !fpcall->GetFunctionSignature()->IsPure())
		{
			mPure = false;
		}

		return mPure;
	}

	bool Visit(wtast::VariableAccess* varacc)
	{
		if(mPure && wtast::AstNode::Variable::GLOBAL==varacc->GetVarClass())
		{
			mPure = false;
		}

		return mPure;
	}
};

class ReturnCoverageVisitor : public wtast::AstVisitor
{
	bool mCovered;

public:
	ReturnCoverageVisitor()
		: wtast::AstVisitor()
		, mCovered(false)
	{
		// ok
	}

	bool IsCovered() const
	{
		return mCovered;
	}

	bool Visit(wtast::ExceptionCompound* excomp)
	{
		if(0<excomp->GetNumberOfStatements())
		{
			excomp->GetStatement(excomp->GetNumberOfStatements()-1)->Accept(this);
		}

		return false;
	}

	bool Visit(wtast::Compound* comp)
	{
		if(0<comp->GetNumberOfStatements())
		{
			comp->GetStatement(comp->GetNumberOfStatements()-1)->Accept(this);
		}

		return false;
	}

	bool Visit(wtast::IfStatement* ifstmt)
	{
		if(ifstmt->GetElse())
		{
			ReturnCoverageVisitor rcv_then;
			ReturnCoverageVisitor rcv_else;

			ifstmt->GetThen()->Accept(&rcv_then);
			ifstmt->GetElse()->Accept(&rcv_else);

			mCovered = rcv_then.mCovered && rcv_else.mCovered;
		}
		else
		{
			//ifstmt->GetThen()->Accept(this);
			mCovered = false;
		}

		return false;
	}

	bool Visit(wtast::ReturnStatement*)
	{
		mCovered = true;
		return false;
	}

	bool Visit(wtast::RaiseStatement*)
	{
		mCovered = true;
		return false;
	}
};

} // visitor

bool SubtreeIsPure(wtast::AstNode* node)
{
	visitor::PurePropVisitor ppv;
	node->Accept(&ppv);
	return ppv.IsPure();
}

bool CheckFunctionReturnCoverage(wtast::FunctionDefinition* funcdef)
{
	visitor::ReturnCoverageVisitor rcv;
	funcdef->Accept(&rcv);
	return rcv.IsCovered();
}

} // checker
} // wtparser
