#include "config_private.h"
#include "vector.h"
#include "vm.h"
#include <string.h>

int wtr_vector_create(struct wtr_vector* const vec, struct wtr_vm* const vm,
	size_t capacity)
{
	if(NULL==vec)
	{
		return 1;
	}

	if(4>capacity)
	{
		capacity = 4;
	}

	memset(vec, 0, sizeof(*vec));

	if(NULL==(vec->data=(void**)wtr_vm_sys_malloc(vm,
		sizeof(void*)*capacity)))
	{
		return 2;
	}

	vec->count    = 0;
	vec->capacity = capacity;

	return 0;
}

void wtr_vector_delete(struct wtr_vector* const vec,
	struct wtr_vm* const vm)
{
	if(NULL!=vec->data)
	{
		wtr_vm_sys_free(vm, vec->data);
	}

	memset(vec, 0, sizeof(*vec));
}

void wtr_vector_clear(struct wtr_vector* const vec)
{
	vec->count = 0;
}

int wtr_vector_push_back(struct wtr_vector* const vec,
	struct wtr_vm* const vm, void* const new_elem)
{
	if(vec->count==vec->capacity)
	{
		void** const new_data = (void**)wtr_vm_sys_realloc(vm, vec->data,
			1*sizeof(void*)*vec->capacity,
			2*sizeof(void*)*vec->capacity);

		if(NULL==new_data)
		{
			return 1;
		}

		vec->data      = new_data;
		vec->capacity *= 2;
	}

	vec->data[vec->count++] = new_elem;

	return 0;
}

int wtr_vector_pop_back(struct wtr_vector* const vec)
{
	if(0<vec->count)
	{
		--vec->count;
		return 0;
	}

	return -1;
}

void** wtr_vector_at(struct wtr_vector* const vec, const size_t index)
{
	if(index<vec->count)
	{
		return &vec->data[index];
	}

	return NULL;
}

int wtr_vector_set(struct wtr_vector* const vec, const size_t index,
	void* const elem)
{
	if(index<vec->count)
	{
		vec->data[index] = elem;
		return 0;
	}

	return 1;
}

int wtr_vector_shrink_to_fit(struct wtr_vector* const vec,
	struct wtr_vm* const vm)
{
	if(0==vec->count)
	{
		if(NULL!=vec->data)
		{
			wtr_vm_sys_free(vm, vec->data);
			vec->data = NULL;
			vec->capacity = 0;
		}
	}
	else
	if(vec->count<vec->capacity)
	{
		void** const new_data = (void**)wtr_vm_sys_realloc(vm, vec->data,
			sizeof(void*)*vec->capacity,
			sizeof(void*)*vec->count);

		if(NULL==new_data)
		{
			return 1;
		}

		vec->data     = new_data;
		vec->capacity = vec->count;
	}

	return 0;
}

int wtr_vector_find(struct wtr_vector* const vec, void* const elem,
	size_t* const out_index)
{
	void**       data = vec->data;
	void** const end  = data + vec->count;

	while(data<end)
	{
		if(elem==(*data))
		{
			if(NULL!=out_index)
			{
				*out_index = data - vec->data;
			}

			return 1;
		}

		++data;
	}

	return 0;
}
